import os
import sys

if sys.version_info >= (3, 12):
    os_path_splitroot = os.path.splitroot

elif os.name ==  'posix':
    # from Lib/posixpath.py
    def os_path_splitroot(p):
        """Split a pathname into drive, root and tail.

        The tail contains anything after the root."""
        p = os.fspath(p)
        if isinstance(p, bytes):
            sep = b'/'
            empty = b''
        else:
            sep = '/'
            empty = ''
        if p[:1] != sep:
            # Relative path, e.g.: 'foo'
            return empty, empty, p
        elif p[1:2] != sep or p[2:3] == sep:
            # Absolute path, e.g.: '/foo', '///foo', '////foo', etc.
            return empty, sep, p[1:]
        else:
            # Precisely two leading slashes, e.g.: '//foo'. Implementation defined per POSIX, see
            # https://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap04.html#tag_04_13
            return empty, p[:2], p[2:]

elif os.name == 'nt':
    # from Lib/ntpath.py
    def os_path_splitroot(p):
        """Split a pathname into drive, root and tail.

        The tail contains anything after the root."""
        p = os.fspath(p)
        if isinstance(p, bytes):
            sep = b'\\'
            altsep = b'/'
            colon = b':'
            unc_prefix = b'\\\\?\\UNC\\'
            empty = b''
        else:
            sep = '\\'
            altsep = '/'
            colon = ':'
            unc_prefix = '\\\\?\\UNC\\'
            empty = ''
        normp = p.replace(altsep, sep)
        if normp[:1] == sep:
            if normp[1:2] == sep:
                # UNC drives, e.g. \\server\share or \\?\UNC\server\share
                # Device drives, e.g. \\.\device or \\?\device
                start = 8 if normp[:8].upper() == unc_prefix else 2
                index = normp.find(sep, start)
                if index == -1:
                    return p, empty, empty
                index2 = normp.find(sep, index + 1)
                if index2 == -1:
                    return p, empty, empty
                return p[:index2], p[index2:index2 + 1], p[index2 + 1:]
            else:
                # Relative path with root, e.g. \Windows
                return empty, p[:1], p[1:]
        elif normp[1:2] == colon:
            if normp[2:3] == sep:
                # Absolute drive-letter path, e.g. X:\Windows
                return p[:2], p[2:3], p[3:]
            else:
                # Relative path with drive, e.g. X:Windows
                return p[:2], empty, p[2:]
        else:
            # Relative path, e.g. Windows
            return empty, empty, p

else:
    raise ImportError('no os specific module found')



import shutil
import tempfile
import unittest
from io import BytesIO
from pathlib import Path
from secrets import token_bytes, token_urlsafe

from backports.zstd import register_shutil, tarfile, zipfile


class TestShutil(unittest.TestCase):
    def setUp(self):
        register_shutil()
        self.tmpdir = tempfile.TemporaryDirectory()
        self.tmppath = Path(self.tmpdir.name)
        (self.tmppath / "src").mkdir()
        (self.tmppath / "src" / "foo").write_text("bar")

    def tearDown(self):
        self.tmpdir.cleanup()

    def test_registration(self):
        self.assertTrue(("zstdtar", "zstd'ed tar-file") in shutil.get_archive_formats())
        self.assertTrue(("zip", "ZIP file") in shutil.get_archive_formats())
        self.assertTrue(
            ("zstdtar", [".tar.zst", ".tzst"], "zstd'ed tar-file")
            in shutil.get_unpack_formats()
        )
        self.assertTrue(("zip", [".zip"], "ZIP file") in shutil.get_unpack_formats())

    def test_make_archive_tar(self):
        shutil.make_archive((self.tmppath / "archive").as_posix(), "zstdtar", self.tmppath)
        with (self.tmppath / "archive.tar.zst").open("rb") as f:
            self.assertEqual(f.read(4), bytes.fromhex("28 b5 2f fd"))

    def test_unpack_archive_tar(self):
        archive_path = self.tmppath / "test.tar.zst"
        raw = token_bytes(1_000)
        raw_name = token_urlsafe(10)
        with tarfile.open(archive_path, "w:zst") as tf:
            ti = tarfile.TarInfo(raw_name)
            ti.size = len(raw)
            tf.addfile(ti, BytesIO(raw))
        out_path = self.tmppath / "test_tar"
        shutil.unpack_archive(archive_path, out_path)
        self.assertEqual((out_path / raw_name).read_bytes(), raw)

    def test_unpack_archive_zip(self):
        archive_path = self.tmppath / "test.zip"
        raw = token_bytes(1_000)
        raw_name = token_urlsafe(10)
        with zipfile.ZipFile(archive_path, "w") as zf:
            zf.writestr(raw_name, raw, zipfile.ZIP_ZSTANDARD)
        out_path = self.tmppath / "test_zip"
        shutil.unpack_archive(archive_path, out_path)
        self.assertEqual((out_path / raw_name).read_bytes(), raw)


if __name__ == "__main__":
    unittest.main()



import shutil
import sys
import unittest
from io import BytesIO
from pathlib import Path
from secrets import token_bytes, token_urlsafe
from tempfile import TemporaryDirectory

if sys.version_info >= (3, 14):
    from compression import zstd
    import tarfile
    import zipfile
else:
    from backports import zstd
    from backports.zstd import register_shutil, tarfile, zipfile
    register_shutil()

if sys.version_info >= (3, 11):
    from typing import assert_type
else:
    assert_type = lambda *_: None

# these tests are simple checks for main use cases
# to make sure they work with the conditional import in 3.14 as well


class TestCompat(unittest.TestCase):
    def test_compress_decompress(self) -> None:
        raw = token_bytes(1_000)
        assert_type(raw, bytes)
        compressed = zstd.compress(raw)
        assert_type(compressed, bytes)
        decompressed = zstd.decompress(compressed)
        assert_type(decompressed, bytes)
        self.assertEqual(decompressed, raw)

    def test_zstdfile(self) -> None:
        raw = token_bytes(1_000)
        fobj = BytesIO()
        with zstd.ZstdFile(fobj, "w") as fzstd:
            fzstd.write(raw)
        self.assertTrue(fobj.tell() > 0)
        fobj.seek(0)
        with zstd.ZstdFile(fobj) as fzstd:
            data = fzstd.read()
            assert_type(data, bytes)
            self.assertEqual(data, raw)
        self.assertTrue(fobj.tell() > 0)

    def test_open(self) -> None:
        raw = token_bytes(1_000)
        fobj = BytesIO()
        with zstd.open(fobj, "w") as fzstd:
            fzstd.write(raw)
        self.assertTrue(fobj.tell() > 0)
        fobj.seek(0)
        with zstd.open(fobj) as fzstd:
            data = fzstd.read()
            assert_type(data, bytes)
            self.assertEqual(data, raw)
        self.assertTrue(fobj.tell() > 0)

    def test_open_binary(self) -> None:
        raw = token_bytes(1_000)
        fobj = BytesIO()
        with zstd.open(fobj, "wb") as fzstd:
            fzstd.write(raw)
        self.assertTrue(fobj.tell() > 0)
        fobj.seek(0)
        with zstd.open(fobj, "rb") as fzstd:
            data = fzstd.read()
            assert_type(data, bytes)
            self.assertEqual(data, raw)
        self.assertTrue(fobj.tell() > 0)

    def test_open_text(self) -> None:
        raw = token_urlsafe(1_000)
        fobj = BytesIO()
        with zstd.open(fobj, "wt") as fzstd:
            fzstd.write(raw)
        self.assertTrue(fobj.tell() > 0)
        fobj.seek(0)
        with zstd.open(fobj, "rt") as fzstd:
            data = fzstd.read()
            assert_type(data, str)
            self.assertEqual(data, raw)
        self.assertTrue(fobj.tell() > 0)

    def test_tarfile(self) -> None:
        raw = token_bytes(1_000)
        raw_name = token_urlsafe(10)
        with TemporaryDirectory() as tmpfile:
            path = Path(tmpfile) / "archive.tar.zst"
            with tarfile.open(path, "w:zst") as tf:
                ti = tarfile.TarInfo(raw_name)
                ti.size = len(raw)
                tf.addfile(ti, BytesIO(raw))

            with tarfile.open(path) as tf:
                self.assertEqual(tf.getnames(), [raw_name])
                extracted = tf.extractfile(raw_name)
                assert extracted is not None  # for type checkers
                with extracted as fobj:
                    self.assertEqual(fobj.read(), raw)

            shutil.unpack_archive(path, tmpfile)
            self.assertEqual((Path(tmpfile) / raw_name).read_bytes(), raw)

    def test_zipfile(self) -> None:
        raw = token_bytes(1_000)
        raw_name = token_urlsafe(10)
        with TemporaryDirectory() as tmpfile:
            path = Path(tmpfile) / "archive.zip"
            with zipfile.ZipFile(path, "w") as zf:
                zf.writestr(raw_name, raw, zipfile.ZIP_ZSTANDARD)

            with zipfile.ZipFile(path) as zf:
                self.assertEqual(zf.namelist(), [raw_name])
                self.assertEqual(zf.read(raw_name), raw)

            shutil.unpack_archive(path, tmpfile)
            self.assertEqual((Path(tmpfile) / raw_name).read_bytes(), raw)

    def test_shutil_make_archive(self) -> None:
        raw = token_bytes(1_000)
        raw_name = token_urlsafe(10)
        with TemporaryDirectory() as tmpfile:
            path_src = Path(tmpfile) / "src"
            path_src.mkdir()
            (path_src / raw_name).write_bytes(raw)

            path_dst = Path(tmpfile) / "archive"
            shutil.make_archive(path_dst.as_posix(), "zstdtar", path_src)

            with path_dst.with_suffix(".tar.zst").open("rb") as f:
                self.assertEqual(f.read(4), bytes.fromhex("28 b5 2f fd"))


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from backports import zstd


@unittest.skipIf(
    os.environ.get("BACKPORTSZSTD_SKIP_EXTENSION_TEST") == "1",
    "BACKPORTSZSTD_SKIP_EXTENSION_TEST set",
)
class TestExtension(unittest.TestCase):
    def test_multithreading_support(self):
        self.assertFalse(zstd.CompressionParameter.nb_workers.bounds() == (0, 0))

    def test_zstd_version(self):
        self.assertEqual(zstd.zstd_version, "1.5.7")
        self.assertEqual(zstd.zstd_version_info, (1, 5, 7))


if __name__ == "__main__":
    unittest.main()



import platform
import sys
import sysconfig
from distutils.unixccompiler import UnixCCompiler
from pathlib import Path

from setuptools import Extension, setup

if not ((3, 10) <= sys.version_info < (3, 14)):
    raise RuntimeError(f"Unsupported Python version: {sys.version}")


# create a LICENSE_zstd.txt file
# wheels distributions needs to ship the license of the zstd library
ROOT_PATH = Path(__file__).parent.absolute()
with (ROOT_PATH / "LICENSE_zstd.txt").open("w") as f:
    f.write(
        "Depending on how it is built, this package may distribute the zstd library,\n"
        "partially or in its integrality, in source or binary form.\n\n"
        "Its license is reproduced below.\n\n"
        "---\n\n"
    )
    f.write((ROOT_PATH / "src" / "c" / "zstd" / "LICENSE").read_text())


UnixCCompiler.src_extensions.append(".S")


_PLATFORM_IS_WIN = sysconfig.get_platform().startswith("win")
_USE_CFFI = platform.python_implementation() == "PyPy"
try:
    sys.argv.remove("--system-zstd")
except ValueError:
    _SYSTEM_ZSTD = False
else:
    _SYSTEM_ZSTD = True


def locate_sources(*sub_paths):
    extensions = "cC" if _PLATFORM_IS_WIN else "cCsS"
    yield from map(str, Path(*sub_paths).rglob(f"*.[{extensions}]"))


def build_extension():
    kwargs = dict(
        sources=[],
        include_dirs=[],
        libraries=[],
        extra_compile_args=[],
        extra_link_args=[],
        define_macros=[
            ("ZSTD_MULTITHREAD", None),  # enable multithreading support
        ],
    )

    if _PLATFORM_IS_WIN:
        kwargs["extra_compile_args"] += ["/Ob3", "/GF", "/Gy"]
    else:
        kwargs["extra_compile_args"] += ["-g0", "-flto"]
        kwargs["extra_link_args"] += ["-g0", "-flto"]

    if _SYSTEM_ZSTD:
        kwargs["libraries"].append("zstd")
    else:
        kwargs["sources"] += [
            *locate_sources("src", "c", "zstd", "lib", "common"),
            *locate_sources("src", "c", "zstd", "lib", "compress"),
            *locate_sources("src", "c", "zstd", "lib", "decompress"),
            *locate_sources("src", "c", "zstd", "lib", "dictBuilder"),
        ]
        kwargs["include_dirs"] += [
            "src/c/zstd/lib",
            "src/c/zstd/lib/common",
            "src/c/zstd/lib/dictBuilder",
        ]

    if _USE_CFFI:
        import cffi

        ffibuilder = cffi.FFI()
        ffibuilder.cdef((ROOT_PATH / "src" / "c" / "cffi" / "cdef.h").read_text())
        ffibuilder.set_source(
            source=(ROOT_PATH / "src" / "c" / "cffi" / "source.c").read_text(),
            module_name="backports.zstd._zstd_cffi",
            **kwargs,
        )
        return ffibuilder.distutils_extension()

    else:
        kwargs["sources"] += [
            *locate_sources("src", "c", "compression_zstd"),
            *locate_sources("src", "c", "compat"),
        ]
        kwargs["include_dirs"] += [
            "src/c/compat",
            "src/c/compression_zstd",
            "src/c/compression_zstd/clinic",
            "src/c/pythoncapi-compat",
        ]
        return Extension(
            name="backports.zstd._zstd",
            **kwargs,
        )


setup(ext_modules=[build_extension()])
